using System;
using System.Resources;
using System.Drawing;
using System.Collections.Generic;
using System.Text;
using PaintDotNet;
using System.Drawing.Imaging;
using System.Reflection;

namespace PaintDotNet.Effects
{
    /// <summary>
    /// Effect class. Implements a gradient fill draw when the effect is called from the Paint.Net code.
    /// </summary>
    [EffectCategory(EffectCategory.Effect)]
    public class FlipEffect : PaintDotNet.Effects.Effect
    {
        public static Bitmap Icon
        {
            get
            {
                return new Bitmap(typeof(FlipEffect), "FlipIcon.png");
            }
        }
        /// <summary>
        /// Base class override, differs in 3.0, so I hope y'all update it!
        /// </summary>
        public FlipEffect()
            : base("Flip", FlipEffect.Icon, System.Windows.Forms.Keys.None, null, EffectDirectives.None, true)
        {
        }

        /// <summary>
        /// Main Render function. Called multiple times to render sections of the overall selected region.
        /// </summary>
        /// <param name="parameters"></param>
        /// <param name="dstArgs"></param>
        /// <param name="srcArgs"></param>
        /// <param name="rois"></param>
        /// <param name="startIndex"></param>
        /// <param name="length"></param>
        public override void Render(EffectConfigToken parameters, PaintDotNet.RenderArgs dstArgs, PaintDotNet.RenderArgs srcArgs, System.Drawing.Rectangle[] rois, int startIndex, int length)
        {

            FlipEffectConfigToken token = (FlipEffectConfigToken)parameters;

            int Direction = token.Type;  // 0=Horizontal, 1=Vertical

            ColorBgra CurrentPixel;
            PdnRegion selectionRegion = EnvironmentParameters.GetSelection(srcArgs.Surface.Bounds);
            Rectangle selection = EnvironmentParameters.GetSelection(srcArgs.Surface.Bounds).GetBoundsInt();
            long CenterX = (long)(((selection.Right - selection.Left) / 2) + selection.Left);
            long CenterY = (long)(((selection.Bottom - selection.Top) / 2) + selection.Top);

            for (int lRectangleIndex = startIndex; lRectangleIndex < startIndex + length; ++lRectangleIndex)
            {
                Rectangle rect = rois[lRectangleIndex];

                for (int y = rect.Top; y < rect.Bottom; y++)
                {
                    for (int x = rect.Left; x < rect.Right; x++)
                    {
                        if (selectionRegion.IsVisible(x, y))
                        {
                            switch (Direction)
                            {
                                case 0:
                                    CurrentPixel = srcArgs.Surface[selection.Right - 1 - (x - selection.Left), y];
                                    break;
                                case 1:
                                    CurrentPixel = srcArgs.Surface[x, selection.Bottom - 1 - (y - selection.Top)];
                                    break;
                                default:
                                    CurrentPixel = srcArgs.Surface[x, y];
                                    break;
                            }
                            dstArgs.Surface[x, y] = CurrentPixel;
                        }
                    }
                }
            }
        }


        /// <summary>
        /// Constructs a token config dialog
        /// </summary>
        /// <returns></returns>
        public override EffectConfigDialog CreateConfigDialog()
        {
            return new FlipEffectConfigDialog();
        }
    }
}